#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include <sstream>
#include "Common.h"
#include "OrderPlacing.h"
#include "OrderPlacingDlg.h"

void COrderPlacingDlg::SetPropertyValue(
                                 ATL::CComPtr<ICQGOrderProperties>& spProperties,
                                 eOrderProperty propertyType,
                                 ATL::CComVariant propertyValue)
{
   // Get property
   ATL::CComPtr<ICQGOrderProperty> spProperty;
   HRESULT hr = spProperties->get_Item(propertyType, &spProperty);
   AssertCOMError(spProperties, hr);

   // Set property's value
   hr = spProperty->put_Value(propertyValue);
   AssertCOMError(spProperty, hr);
}

void COrderPlacingDlg::ChangePrice(
                                 CEdit& editCtrl,
                                 ATL::CComPtr<ICQGInstrument> spInstrument,
                                 int delta)
{
   // Get price from edit control
   double price = GetPrice(editCtrl, spInstrument);

   if (price == m_INVALID_DOUBLE)
   {
      // Price is invalid
      editCtrl.SetWindowText(m_N_A);
      return;
   }

   ATL::CComPtr<ICQGInstrumentProperties> spProperties;
   HRESULT hr = spInstrument->get_Properties(&spProperties);
   AssertCOMError(spInstrument, hr);

   // Check whether an instrument has a variable tick size
   ATL::CComPtr<ICQGInstrumentProperty> spProperty;
   hr = spProperties->get_Item(ipIsVariableTickSize, &spProperty);
   AssertCOMError(spProperties, hr);

   ATL::CComVariant isVariableTickSize{ false };
   spProperty->get_Value(&isVariableTickSize);

   // Get tick size from instrument
   double tickSize{};
   hr = isVariableTickSize.boolVal == VARIANT_TRUE ? 
      spInstrument->GetTickSizeFor(price, &tickSize) : spInstrument->get_TickSize(&tickSize);
   AssertCOMError(spInstrument, hr);

   // Add tick size to price, minus is coming from spin button delta
   price -= tickSize * delta;

   // Get new price display format
   ATL::CComBSTR displayPriceBSTR;
   hr = spInstrument->ToDisplayPrice(price, &displayPriceBSTR);
   AssertCOMError(spInstrument, hr);

   // Show new price in edit control
   CString displayPrice = ATL::CW2T(displayPriceBSTR);

   editCtrl.SetWindowText(displayPrice);
}

double COrderPlacingDlg::GetPrice(
                                 CEdit& editCtrl,
                                 ATL::CComPtr<ICQGInstrument> spInstrument)
{
   // Get price display format from edit control
   CString priceStr;
   editCtrl.GetWindowText(priceStr);

   if (priceStr == m_N_A || priceStr.GetLength() == 0)
   {
      // Price is invalid
      return m_INVALID_DOUBLE;
   }

   // Get price from display price
   double price;
   HRESULT hr = spInstrument->FromDisplayPrice(ATL::CT2W(priceStr), &price);
   AssertCOMError(spInstrument, hr);
   
   return price;
}

DECIMAL COrderPlacingDlg::GetQuantity(
                                 CEdit& editCtrl)
{
   CString quantityText;
   editCtrl.GetWindowText(quantityText);

   _bstr_t quantityStr{ quantityText };

   DECIMAL quantity;
   if (FAILED(VarDecFromStr(quantityStr, LOCALE_ILANGUAGE, LOCALE_NOUSEROVERRIDE, &quantity)))
   {
      throw std::runtime_error("Couldn't get quantity.");
   }

   return quantity;
}

DECIMAL COrderPlacingDlg::GetValidQuantity(
                                 DECIMAL quantity)
{
   DECIMAL validQuantity;

   HRESULT hr = S_OK;

   // Validate quantity for maximum and minimum values
   DECIMAL minQuantity = m_MIN_QUANTITY;
   DECIMAL maxQuantity = m_MAX_QUANTITY;
   if (VARCMP_LT == VarDecCmp(&quantity, &minQuantity))
   {
      validQuantity = m_MIN_QUANTITY;
   }
   else if (VARCMP_GT == VarDecCmp(&quantity, &maxQuantity))
   {
      validQuantity = m_MAX_QUANTITY;
   }
   else
   {
      validQuantity = quantity;
   }

   if (hr != S_OK)
   {
      throw std::runtime_error("Couldn't set min or max value for quantity.");
   }

   return validQuantity;
}

void COrderPlacingDlg::SetQuantity(
                                 CEdit& editCtrl,
                                 DECIMAL quantity)
{
   editCtrl.SetWindowText(_bstr_t(VariantToString(_variant_t(quantity))));
}

void COrderPlacingDlg::SpinQuantity(
                                 CEdit& editCtrl,
                                 NMHDR* pNMHDR)
{
   LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);

   DECIMAL quantity = GetQuantity(editCtrl);

   // We increment/decrement number with given scale (e.g.: 1 -> 2, 0.02 -> 0.01),
   // unless our decrement should go up in scale (e.g.: 1 -> 0.9, 0.01 -> 0.009).
   // Increment is not changed, it will always increment to lesser scale.
   // Sometimes you may need numbers like 0.0175, for such cases edit box allows entering numbers directly.
   DECIMAL delta;
   VarDecFromInt(pNMUpDown->iDelta, &delta);
   delta.scale = quantity.scale;

   DECIMAL result;
   VarDecSub(&quantity, &delta, &result);

   DECIMAL zeroQuantity;
   DECIMAL_SETZERO(zeroQuantity);
   if (VARCMP_EQ == VarDecCmp(&result, &zeroQuantity) && quantity.scale >= 0)
   {
      delta.scale = quantity.scale + 1;
      VarDecSub(&quantity, &delta, &result);
   }

   SetQuantity(editCtrl, GetValidQuantity(result));
}
